# $Id: raterecord.py 3919 2010-08-31 06:51:38Z place $
# $URL: https://svn.non-gnu.uvt.nl/uvt-dev/tags/sources/meresco/bzv-9.9.0/src/delfttilburg/raterecord.py $
## begin license ##
#
#    "Delft-Tilburg" (delfttilburg) is a package containing shared code
#    between the Delft "Discover" and Tilburg "Beter Zoeken & Vinden" projects.
#    Both projects are based on Meresco Software (http://meresco.com)
#    Copyright (C) 2008-2009 Technische Universiteit Delft http://www.tudelft.nl
#    Copyright (C) 2008-2009 Universiteit van Tilburg http://www.uvt.nl
#    Copyright (C) 2008-2009 Seek You Too (CQ2) http://www.cq2.nl
#
#    This file is part of "Delft-Tilburg".
#
#    "Delft-Tilburg" is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    "Delft-Tilburg" is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with "Delft-Tilburg"; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
## end license ##
from meresco.core import Observable

from lxml.etree import parse
from StringIO import StringIO

RATING_TEMPLATE = """<rating xmlns="http://meresco.com/namespace/record/rating">
    <numberOfVotes>%d</numberOfVotes>
    <totalScore>%d</totalScore>
    <averageScore>%.0f</averageScore>
</rating>"""

EMPTY_RATING = RATING_TEMPLATE % (0, 0, 0)

NSMAP = {'rating': "http://meresco.com/namespace/record/rating"}

class RateRecord(Observable):
    def __init__(self, minRating=1, maxRating=5):
        Observable.__init__(self)
        self._minRating = minRating
        self._maxRating = maxRating

    def rateRecord(self, recordId, rateValue):
        rateValue = max(self._minRating, min(self._maxRating, rateValue))

        ignored, partExists = self.any.isAvailable(recordId, 'rating')
        oldRating = partExists and self.any.getStream(recordId, 'rating') or StringIO(EMPTY_RATING)

        xmlNode = parse(oldRating)
        numberOfVotes = int(xmlNode.xpath('/rating:rating/rating:numberOfVotes/text()', namespaces=NSMAP)[0])
        totalScore = int(xmlNode.xpath('/rating:rating/rating:totalScore/text()', namespaces=NSMAP)[0])

        numberOfVotes = numberOfVotes + 1
        totalScore = totalScore + rateValue
        averageScore = float(totalScore) / float(numberOfVotes) * 1000

        newRecord = parse(StringIO(RATING_TEMPLATE % (numberOfVotes, totalScore, averageScore)))

        self.do.add(recordId, 'rating', newRecord)
